@@ -7,6 +7,7 @@
 #include <unicase.h>
 #include <unictype.h>
 #include <uniname.h>
+#include <uninorm.h>
 #include <unistr.h>
 #include "confusables.h"
 
@@ -16,14 +17,6 @@
 static const uc_block_t *all_blocks;
 static size_t all_block_count;
 
-/* libunistring <= 0.9.4 doesn't have Emoticons block defined.  */
-static const uc_block_t emoticon_block =
-  {
-    0x1F600,
-    0x1F64F,
-    "Emoticons"
-  };
-
 /* Bullets are not specially categorized in the Unicode standard.
    Use the character list from UTR#25 "Unicode Support for Mathematics".  */
 static const gunichar bullet_characters[] =
@@ -279,7 +272,7 @@ gc_enumerate_character_by_category (GcCharacterIter *iter,
 
     case GC_CATEGORY_PICTURE:
       {
-	static uc_block_t picture_blocks[2];
+	static uc_block_t picture_blocks[6];
 	static gsize picture_blocks_size = 0;
 	static gsize picture_blocks_initialized = 0;
 	if (g_once_init_enter (&picture_blocks_initialized))
@@ -293,6 +286,26 @@ gc_enumerate_character_by_category (GcCharacterIter *iter,
 		      sizeof (uc_block_t));
 	    /* 2700..27BF; Dingbats */
 	    block = uc_block (0x2700);
+	    if (block)
+	      memcpy (&picture_blocks[picture_blocks_size++], block,
+		      sizeof (uc_block_t));
+	    /* 1F000..1F02F; Mahjong Tiles */
+	    block = uc_block (0x1F000);
+	    if (block)
+	      memcpy (&picture_blocks[picture_blocks_size++], block,
+		      sizeof (uc_block_t));
+	    /* 1F030..1F09F; Domino Tiles */
+	    block = uc_block (0x1F030);
+	    if (block)
+	      memcpy (&picture_blocks[picture_blocks_size++], block,
+		      sizeof (uc_block_t));
+	    /* 1F0A0..1F0FF; Playing Cards */
+	    block = uc_block (0x1F0A0);
+	    if (block)
+	      memcpy (&picture_blocks[picture_blocks_size++], block,
+		      sizeof (uc_block_t));
+	    /* 1F300..1F5FF; Miscellaneous Symbols and Pictographs */
+	    block = uc_block (0x1F300);
 	    if (block)
 	      memcpy (&picture_blocks[picture_blocks_size++], block,
 		      sizeof (uc_block_t));
@@ -328,21 +341,15 @@ gc_enumerate_character_by_category (GcCharacterIter *iter,
 
 	    /* 1F600..1F64F; Emoticons */
 	    block = uc_block (0x1F600);
-
-	    /* libunistring <= 0.9.4 doesn't have Emoticons block defined.  */
-	    if (!block)
-	      block = &emoticon_block;
-
-	    memcpy (&emoticon_blocks[emoticon_blocks_size++], block,
-		    sizeof (uc_block_t));
+	    if (block)
+	      memcpy (&emoticon_blocks[emoticon_blocks_size++], block,
+		      sizeof (uc_block_t));
 	    g_once_init_leave (&emoticon_blocks_initialized, 1);
 	  }
 	gc_character_iter_init_for_blocks (iter,
 					   emoticon_blocks,
 					   emoticon_blocks_size);
-	/* libunistring <= 0.9.4 doesn't have Emoticons block defined
-	   and we can't use uc_is_print() here.  */
-	iter->filter = filter_all;
+	iter->filter = filter_is_print;
 	return;
       }
     }
@@ -610,6 +617,40 @@ remove_duplicates (GArray *array)
     }
 }
 
+static void
+add_composited (GArray *result,
+		ucs4_t uc,
+		ucs4_t *block_characters,
+		size_t n_block_characters)
+{
+  ucs4_t decomposition[UC_DECOMPOSITION_MAX_LENGTH];
+  int decomposition_length;
+  ucs4_t base;
+  size_t i;
+
+  decomposition_length = uc_canonical_decomposition (uc, decomposition);
+  if (decomposition_length > 0)
+    {
+      base = decomposition[0];
+      g_array_append_val (result, base);
+    }
+  else
+    base = uc;
+
+  for (i = 0; i < n_block_characters; i++)
+    {
+      const uc_block_t *block;
+
+      block = uc_block (block_characters[i]);
+      for (uc = block->start; uc < block->end; uc++)
+	{
+	  decomposition_length = uc_canonical_decomposition (uc, decomposition);
+	  if (decomposition_length > 0 && decomposition[0] == base)
+	    g_array_append_val (result, uc);
+	}
+    }
+}
+
 static void
 gc_search_related_thread (GTask        *task,
 			  gpointer      source_object,
@@ -639,6 +680,34 @@ gc_search_related_thread (GTask        *task,
       g_array_append_val (result, uc);
     }
 
+  if (uc_is_general_category (data->uc, UC_CATEGORY_L))
+    {
+      const uc_script_t *script;
+
+      script = uc_script (data->uc);
+      if (script)
+	{
+	  if (strcmp (script->name, "Latin") == 0)
+	    {
+	      ucs4_t block_starters[4] = { 0x0000, 0x0080, 0x0100, 0x0180 };
+	      add_composited (result, data->uc,
+			      block_starters, G_N_ELEMENTS (block_starters));
+	    }
+	  else if (strcmp (script->name, "Hiragana") == 0)
+	    {
+	      ucs4_t block_starters[1] = { 0x3040 };
+	      add_composited (result, data->uc,
+			      block_starters, G_N_ELEMENTS (block_starters));
+	    }
+	  else if (strcmp (script->name, "Katakana") == 0)
+	    {
+	      ucs4_t block_starters[2] = { 0x30A0, 0x31F0 };
+	      add_composited (result, data->uc,
+			      block_starters, G_N_ELEMENTS (block_starters));
+	    }
+	}
+    }
+
   gc_add_confusables (data, result, cancellable);
   g_array_sort (result, compare_unichar);
   remove_duplicates (result);
