@@ -93,7 +93,8 @@ const CategoryListRowWidget = new Lang.Class({
         this.get_accessible().accessible_name =
             _('%s Category List Row').format(category.title);
 
-        let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL });
+        let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL,
+                                 margin_start: 10 });
         this.add(hbox);
 
         let icon = new Gio.ThemedIcon({ name: category.icon_name });
@@ -116,9 +117,15 @@ const CategoryListWidget = new Lang.Class({
         params = Params.fill(params, {});
         this.parent(params);
 
+        // Mimic GtkStackSidebar to take advantage of the standard theme.
+        this.get_style_context().add_class('sidebar');
+
         for (let index in Category) {
             let category = Category[index];
-            this.add(new CategoryListRowWidget({}, category));
+            let rowWidget = new CategoryListRowWidget({}, category);
+            // Mimic GtkStackSidebar to take advantage of the standard theme.
+            rowWidget.get_style_context().add_class('sidebar-item');
+            this.add(rowWidget);
         }
     },
 
