@@ -128,8 +128,11 @@ const MainWindow = new Lang.Class({
         if (this._searchActive == v)
             return;
 
-        this._categoryList.unselect_all();
         this._searchActive = v;
+
+        if (this._searchActive)
+            this._categoryList.unselect_all();
+
         this.notify('search-active');
     },
 
@@ -155,13 +158,16 @@ const MainWindow = new Lang.Class({
 
     _about: function() {
         let aboutDialog = new Gtk.AboutDialog(
-            { authors: [ 'Daiki Ueno <dueno@src.gnome.org>' ],
+            { artists: [ 'Allan Day <allanpday@gmail.com>',
+                         'Jakub Steiner <jimmac@gmail.com>' ],
+              authors: [ 'Daiki Ueno <dueno@src.gnome.org>' ],
+              // TRANSLATORS: put your names here, one name per line.
               translator_credits: _("translator-credits"),
               program_name: _("GNOME Characters"),
               comments: _("Character Map"),
               copyright: 'Copyright 2014-2015 Daiki Ueno',
               license_type: Gtk.License.GPL_2_0,
-              logo_icon_name: 'org.gnome.Characters',
+              logo_icon_name: 'gnome-characters',
               version: pkg.version,
               // website: 'https://wiki.gnome.org/Design/Apps/CharacterMap',
               wrap_license: true,
@@ -186,6 +192,8 @@ const MainWindow = new Lang.Class({
     },
 
     _category: function(action, v) {
+        this.search_active = false;
+
         let [name, length] = v.get_string()
 
         // FIXME: we could use Gtk.Container.get_child to obtain the
@@ -215,6 +223,11 @@ const MainWindow = new Lang.Class({
         this._filterFontFamily = family;
         this._updateTitle(this._mainView.visible_child.title);
     },
+
+    setSearchKeywords: function(keywords) {
+        this.search_active = keywords.length > 0;
+        this._search_entry.set_text(keywords.join(' '));
+    }
 });
 
 const MainView = new Lang.Class({
@@ -239,7 +252,10 @@ const MainView = new Lang.Class({
     },
 
     _init: function(params) {
-        params = Params.fill(params, { hexpand: true, vexpand: true });
+        params = Params.fill(params, {
+            hexpand: true, vexpand: true,
+            transition_type: Gtk.StackTransitionType.CROSSFADE
+        });
         this.parent(params);
 
         this._characterLists = {};
@@ -295,15 +311,15 @@ const MainView = new Lang.Class({
         if (!(name in this._characterLists))
             return;
 
-        this.visible_child_name = name;
-        this.visible_child.setFilterFont(filterFontFamily);
+        let characterList = this.get_child_by_name(name);
+        characterList.setFilterFont(filterFontFamily);
 
         if (name == 'recent') {
             if (this.recentCharacters.length == 0)
-                this.visible_child.visible_child_name = 'empty-recent';
+                characterList.visible_child_name = 'empty-recent';
             else {
-                this.visible_child.setCharacters(this.recentCharacters);
-                this.visible_child.updateCharacterList();
+                characterList.setCharacters(this.recentCharacters);
+                characterList.updateCharacterList();
             }
         } else {
             let category = null;
@@ -314,8 +330,10 @@ const MainView = new Lang.Class({
             }
 
             Util.assertNotEqual(category, null);
-            this.visible_child.searchByCategory(category);
+            characterList.searchByCategory(category);
         }
+
+        this.visible_child = characterList;
     },
 
     addToRecent: function(uc) {
